/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;
import org.fortiss.tooling.kernel.extension.exception.TransformationFailedException;
import org.fortiss.tooling.kernel.service.ITransformationService;
import org.fortiss.tooling.kernel.service.base.IObjectAware2;

/**
 * A transformation provider represents a concrete transformation from a source
 * element into a target element and is registered with the
 * {@link ITransformationService}'s extension point. Since transformations may
 * also produce intermediary data, which is not intended to be persisted, this
 * class uses {@link Object} instead of {@link EObject}s only.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D798E691812994DC6321748E8CAC1C20
 */
public interface ITransformationProvider extends IObjectAware2<Object, Object> {

	/** Returns the target class this provider can provide. */
	Class<?> getTargetClass();

	/**
	 * Returns whether this provider is able to become part of a chain,
	 * transforming objects of the given source class into the provider's target
	 * class. Note that a provider may return <code>true</code> here and return
	 * <code>false</code> later when its
	 * {@link #canTransform(Object, ITransformationContext)} is called and the
	 * concrete source object to be transformed cannot be handled.
	 */
	boolean canHandleChainTransformation(Class<?> sourceClass,
			ITransformationContext context);

	/**
	 * Returns whether the transformation provider can transform the given
	 * source element using the provided transformation context element.
	 */
	boolean canTransform(Object source, ITransformationContext context);

	/**
	 * Returns the transformation result. This method is only called if
	 * {@link #canTransform(Object, ITransformationContext)} returns
	 * <code>true</code>. This method may return <code>null</code> if the
	 * transformation was aborted by, e.g., the user. It may also throw a
	 * {@link TransformationFailedException} if something goes wrong.
	 */
	Object transform(Object source, ITransformationContext context)
			throws TransformationFailedException;
}
